@@ -62,31 +62,12 @@ impl Bitboard {
         Bitboard(RANKS[usize::from(rank)])
     }
 
-    #[deprecated(since = "0.20.4", note = "Renamed to Bitboard::from_rank()")]
-    #[inline]
-    pub fn rank(rank: Rank) -> Bitboard {
-        Bitboard::from_rank(rank)
-    }
-
     /// Returns the bitboard containing all squares of the given file.
     #[inline]
     pub fn from_file(file: File) -> Bitboard {
         Bitboard(FILES[usize::from(file)])
     }
 
-    #[deprecated(since = "0.20.4", note = "Renamed to Bitboard::from_file()")]
-    #[inline]
-    pub fn file(file: File) -> Bitboard {
-        Bitboard::from_file(file)
-    }
-
-    /// Like `rank()`, but from the point of view of `color`.
-    #[deprecated(since = "0.20.4", note = "Use color.relative_rank()")]
-    #[inline]
-    pub fn relative_rank(color: Color, rank: Rank) -> Bitboard {
-        Bitboard::from_rank(color.fold_wb(rank, rank.flip_vertical()))
-    }
-
     /// Shift using `<<` for `White` and `>>` for `Black`.
     ///
     /// # Examples
@@ -177,7 +158,7 @@ impl Bitboard {
     /// assert_eq!(bitboard.remove(Square::E4), true);
     /// assert_eq!(bitboard.remove(Square::E4), false);
     /// ```
-    #[must_use]
+    #[must_use = "use Bitboard::discard() if return value is not needed"]
     #[inline]
     pub fn remove(&mut self, sq: Square) -> bool {
         if self.contains(sq) {
@@ -698,9 +679,6 @@ impl Bitboard {
     /// // . . . . 1 1 1 1
     /// ```
     pub const EAST: Bitboard = Bitboard(0xf0f0_f0f0_f0f0_f0f0);
-
-    #[deprecated(since = "0.20.1", note = "Renamed to Bitboard::FULL for consistency")]
-    pub const ALL: Bitboard = Bitboard::FULL;
 }
 
 /// Square masks.
