@@ -14,11 +14,14 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-use std::{error::Error, fmt, iter::FusedIterator, mem, ops, str::FromStr};
+//! White or black.
+
+use std::{array, error::Error, fmt, mem, ops, str::FromStr};
 
 use crate::{
+    role::{ByRole, Role},
     square::Rank,
-    types::{Piece, Role},
+    types::Piece,
 };
 
 /// `White` or `Black`.
@@ -31,11 +34,27 @@ pub enum Color {
 
 impl Color {
     pub fn from_char(ch: char) -> Option<Color> {
-        match ch {
-            'w' => Some(Color::White),
-            'b' => Some(Color::Black),
-            _ => None,
-        }
+        Some(match ch {
+            'w' => Color::White,
+            'b' => Color::Black,
+            _ => return None,
+        })
+    }
+
+    pub fn char(self) -> char {
+        self.fold_wb('w', 'b')
+    }
+
+    fn from_name(name: &str) -> Option<Color> {
+        Some(match name {
+            "white" => Color::White,
+            "black" => Color::Black,
+            _ => return None,
+        })
+    }
+
+    fn name(self) -> &'static str {
+        self.fold_wb("white", "black")
     }
 
     #[inline]
@@ -56,12 +75,6 @@ impl Color {
         }
     }
 
-    #[deprecated = "use Color::fold_wb instead"]
-    #[inline]
-    pub fn fold<T>(self, white: T, black: T) -> T {
-        self.fold_wb(white, black)
-    }
-
     #[inline]
     pub fn fold_wb<T>(self, white: T, black: T) -> T {
         match self {
@@ -92,10 +105,6 @@ impl Color {
         }
     }
 
-    pub fn char(self) -> char {
-        self.fold_wb('w', 'b')
-    }
-
     #[inline]
     pub fn pawn(self) -> Piece {
         Role::Pawn.of(self)
@@ -145,7 +154,7 @@ impl ops::BitXor<bool> for Color {
 
 impl fmt::Display for Color {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.write_str(self.fold_wb("white", "black"))
+        f.write_str(self.name())
     }
 }
 
@@ -165,19 +174,16 @@ impl FromStr for Color {
     type Err = ParseColorError;
 
     fn from_str(s: &str) -> Result<Color, ParseColorError> {
-        Ok(match s {
-            "black" => Color::Black,
-            "white" => Color::White,
-            _ => return Err(ParseColorError),
-        })
+        Color::from_name(s).ok_or(ParseColorError)
     }
 }
 
 /// Container with values for each [`Color`].
 #[derive(Clone, Default, Eq, PartialEq, Debug, Hash)]
+#[repr(C)]
 pub struct ByColor<T> {
-    pub white: T,
     pub black: T,
+    pub white: T,
 }
 
 impl<T> ByColor<T> {
@@ -193,27 +199,19 @@ impl<T> ByColor<T> {
     }
 
     #[inline]
-    pub fn by_color(&self, color: Color) -> &T {
-        match color {
-            Color::White => &self.white,
-            Color::Black => &self.black,
+    pub fn get(&self, color: Color) -> &T {
+        // Safety: Trivial offset into #[repr(C)] struct.
+        unsafe {
+            &*(self as *const ByColor<T>)
+                .cast::<T>()
+                .offset(color as isize)
         }
     }
 
     #[inline]
-    pub fn by_color_mut(&mut self, color: Color) -> &mut T {
-        match color {
-            Color::White => &mut self.white,
-            Color::Black => &mut self.black,
-        }
-    }
-
-    #[inline]
-    pub fn into_color(self, color: Color) -> T {
-        match color {
-            Color::White => self.white,
-            Color::Black => self.black,
-        }
+    pub fn get_mut(&mut self, color: Color) -> &mut T {
+        // Safety: Trivial offset into #[repr(C)] struct.
+        unsafe { &mut *(self as *mut ByColor<T>).cast::<T>().offset(color as isize) }
     }
 
     pub fn flip(&mut self) {
@@ -223,11 +221,20 @@ impl<T> ByColor<T> {
     #[must_use]
     pub fn into_flipped(self) -> ByColor<T> {
         ByColor {
-            white: self.black,
             black: self.white,
+            white: self.black,
         }
     }
 
+    #[inline]
+    pub fn for_each<F>(self, mut f: F)
+    where
+        F: FnMut(T),
+    {
+        f(self.white);
+        f(self.black);
+    }
+
     #[inline]
     pub fn map<U, F>(self, mut f: F) -> ByColor<U>
     where
@@ -239,22 +246,6 @@ impl<T> ByColor<T> {
         }
     }
 
-    #[inline]
-    pub fn any<F>(&self, mut predicate: F) -> bool
-    where
-        F: FnMut(&T) -> bool,
-    {
-        predicate(&self.white) || predicate(&self.black)
-    }
-
-    #[inline]
-    pub fn all<F>(&self, mut predicate: F) -> bool
-    where
-        F: FnMut(&T) -> bool,
-    {
-        predicate(&self.white) && predicate(&self.black)
-    }
-
     #[inline]
     pub fn find<F>(&self, mut predicate: F) -> Option<Color>
     where
@@ -272,42 +263,59 @@ impl<T> ByColor<T> {
     #[inline]
     pub fn as_ref(&self) -> ByColor<&T> {
         ByColor {
-            white: &self.white,
             black: &self.black,
+            white: &self.white,
         }
     }
 
     #[inline]
     pub fn as_mut(&mut self) -> ByColor<&mut T> {
         ByColor {
-            white: &mut self.white,
             black: &mut self.black,
+            white: &mut self.white,
         }
     }
 
     pub fn zip_color(self) -> ByColor<(Color, T)> {
         ByColor {
-            white: (Color::White, self.white),
             black: (Color::Black, self.black),
+            white: (Color::White, self.white),
         }
     }
 
     pub fn zip<U>(self, other: ByColor<U>) -> ByColor<(T, U)> {
         ByColor {
-            white: (self.white, other.white),
             black: (self.black, other.black),
+            white: (self.white, other.white),
         }
     }
 
-    pub fn iter(&self) -> ByColorIter<&T> {
+    pub fn iter(&self) -> array::IntoIter<&T, 2> {
         self.as_ref().into_iter()
     }
 
-    pub fn iter_mut(&mut self) -> ByColorIter<&mut T> {
+    pub fn iter_mut(&mut self) -> array::IntoIter<&mut T, 2> {
         self.as_mut().into_iter()
     }
 }
 
+impl<T> ByColor<ByRole<T>> {
+    pub fn piece(&self, piece: Piece) -> &T {
+        self.get(piece.color).get(piece.role)
+    }
+
+    pub fn piece_mut(&mut self, piece: Piece) -> &mut T {
+        self.get_mut(piece.color).get_mut(piece.role)
+    }
+}
+
+#[cfg(feature = "variant")]
+impl ByColor<ByRole<u8>> {
+    pub(crate) fn count(&self) -> usize {
+        self.iter().map(|side| side.count()).sum()
+    }
+}
+
 impl<T: PartialOrd> ByColor<T> {
     pub fn normalize(&mut self) {
         if self.white < self.black {
@@ -330,63 +338,21 @@ impl<T: PartialEq> ByColor<T> {
 
 impl<T: Copy> ByColor<&T> {
     pub fn copied(self) -> ByColor<T> {
-        ByColor {
-            white: *self.white,
-            black: *self.black,
-        }
+        self.map(|item| *item)
     }
 }
 
 impl<T: Clone> ByColor<&T> {
     pub fn cloned(self) -> ByColor<T> {
-        ByColor {
-            white: self.white.clone(),
-            black: self.black.clone(),
-        }
+        self.map(Clone::clone)
     }
 }
 
 impl<T> IntoIterator for ByColor<T> {
     type Item = T;
-    type IntoIter = ByColorIter<T>;
+    type IntoIter = array::IntoIter<T, 2>;
 
-    fn into_iter(self) -> ByColorIter<T> {
-        ByColorIter {
-            inner: self.map(Some),
-        }
+    fn into_iter(self) -> Self::IntoIter {
+        [self.white, self.black].into_iter()
     }
 }
-
-/// Iterator over [`ByColor`].
-#[derive(Debug, Clone)]
-pub struct ByColorIter<T> {
-    inner: ByColor<Option<T>>,
-}
-
-impl<T> Iterator for ByColorIter<T> {
-    type Item = T;
-
-    fn next(&mut self) -> Option<T> {
-        self.inner.white.take().or_else(|| self.inner.black.take())
-    }
-
-    fn size_hint(&self) -> (usize, Option<usize>) {
-        let len = self.len();
-        (len, Some(len))
-    }
-}
-
-impl<T> ExactSizeIterator for ByColorIter<T> {
-    fn len(&self) -> usize {
-        (if self.inner.white.is_some() { 1 } else { 0 })
-            + (if self.inner.black.is_some() { 1 } else { 0 })
-    }
-}
-
-impl<T> DoubleEndedIterator for ByColorIter<T> {
-    fn next_back(&mut self) -> Option<T> {
-        self.inner.black.take().or_else(|| self.inner.white.take())
-    }
-}
-
-impl<T> FusedIterator for ByColorIter<T> {}
