@@ -16,201 +16,131 @@
 
 //! Parse and write Forsyth-Edwards-Notation.
 //!
-//! # En passant only if fully legal
+//! # Parsing
 //!
-//! This implementation intentionally deviates from the specification in
-//! a backwards compatible way: [`Setup::ep_square()`]
-//! implemented for each [`Position`](crate::Position) omits en passant
-//! squares, unless there is a fully legal en passant capture.
+//! The parser is relaxed:
 //!
-//! # Examples
+//! * Supports X-FEN and Shredder-FEN for castling right notation.
+//!   Allows repeated castling rights and castling rights in any order.
+//! * Supports `[q]` and `/q` styles for Crazyhouse pockets.
+//! * Supports `3+3` and `+0+0` for remaining checks in Three-Check.
+//! * Accepts partial FENs and fills missing fields with the default values
+//!   of `8/8/8/8/8/8/8/8 w - - 0 1`.
+//! * Accepts multiple spaces and underscores (`_`) as separators between
+//!   FEN fields.
+//! * Accepts `0` as fulllmove number and uses `1` instead.
 //!
-//! [`fen::fen()`](fen()) and [`fen::epd()`](epd()) can produce a
-//! FEN for any [`Setup`].
+//! [`Fen`] and [`Epd`] implement [`FromStr`]:
 //!
 //! ```
-//! use shakmaty::{fen, Chess};
+//! use shakmaty::{fen::Fen, CastlingMode, Chess, Position};
 //!
-//! let pos = Chess::default();
+//! let fen: Fen = "r1bqkbnr/ppp2Qpp/2np4/4p3/2B1P3/8/PPPP1PPP/RNB1K1NR b KQkq - 0 4".parse()?;
 //!
-//! assert_eq!(fen::epd(&pos),
-//!            "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -");
-//! ```
-//!
-//! [`Fen`] and [`Board`] also implement [`Display`]:
-//!
-//! ```
-//! use shakmaty::fen::Fen;
-//!
-//! let empty_fen = Fen::empty();
-//! assert_eq!(empty_fen.to_string(), "8/8/8/8/8/8/8/8 w - - 0 1");
-//! ```
-//!
-//! Parsing FENs:
-//!
-//! ```
-//! # use shakmaty::{fen::Fen, Chess};
-//! use shakmaty::{CastlingMode, Position};
-//!
-//! let input = "r1bqkbnr/ppp2Qpp/2np4/4p3/2B1P3/8/PPPP1PPP/RNB1K1NR b KQkq - 0 4";
-//!
-//! let setup: Fen = input.parse()?;
-//! let position: Chess = setup.position(CastlingMode::Standard)?;
-//! assert!(position.is_checkmate());
+//! let pos: Chess = fen.into_position(CastlingMode::Standard)?;
+//! assert!(pos.is_checkmate());
 //! # Ok::<_, Box<dyn std::error::Error>>(())
 //! ```
 //!
-//! [`Setup`]: super::Setup
-//! [`Board`]: super::Board
-//! [`Display`]: std::fmt::Display
+//! # Writing
+//!
+//! Writes X-FEN with `[q]` style for Crazyhouse pockets and `3+3` style
+//! for remainig checks in Three-Check.
+//!
+//! [`Fen`] and [`Epd`] implement [`Display`]:
+//!
+//! ```
+//! use shakmaty::{fen::Epd, EnPassantMode, Chess, Position};
+//!
+//! let pos = Chess::default();
+//!
+//! assert_eq!(Epd::from_position(pos, EnPassantMode::Legal).to_string(),
+//!            "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -");
+//! ```
 
-use std::{char, cmp::max, convert::TryFrom, error::Error, fmt, num::NonZeroU32, str::FromStr};
-
-use crate::{
-    bitboard::Bitboard,
-    board::Board,
-    color::{ByColor, Color},
-    material::Material,
-    position::{FromSetup, PositionError},
-    setup::Setup,
-    square::{File, Rank, Square},
-    types::{CastlingMode, Piece, RemainingChecks},
+use std::{
+    char,
+    cmp::max,
+    convert::TryFrom,
+    error::Error,
+    fmt,
+    fmt::{Display, Write as _},
+    num::NonZeroU32,
+    str::FromStr,
 };
 
-/// FEN formatting options.
-#[derive(Default, Clone, Debug)]
-pub struct FenOpts {
-    shredder: bool,
-    scid: bool,
-}
+use crate::{
+    Bitboard, Board, ByColor, ByRole, CastlingMode, Color, File, FromSetup, Piece, PositionError,
+    Rank, RemainingChecks, Role, Setup, Square, EnPassantMode,
+    Position,
+};
 
-impl FenOpts {
-    /// Default X-FEN formatting.
-    pub fn new() -> FenOpts {
-        FenOpts::default()
-    }
+fn fmt_castling(
+    f: &mut fmt::Formatter<'_>,
+    board: &Board,
+    castling_rights: Bitboard,
+) -> fmt::Result {
+    let mut empty = true;
 
-    /// Decide if castling rights should be displayed in Shredder format,
-    /// e.g. `HAha` instead of `KQkq`.
-    ///
-    /// Defaults to `false`.
-    pub fn shredder(&mut self, shredder: bool) -> &mut FenOpts {
-        self.shredder = shredder;
-        self
-    }
+    for color in Color::ALL {
+        let king = board.king_of(color);
 
-    /// Decide if Crazyhouse pockets and remaining check counters should use
-    /// Scid-style, e.g. `/q` instead of `[q]` and `+0+0` instead of `3+3`.
-    ///
-    /// Defaults to `false`.
-    pub fn scid(&mut self, scid: bool) -> &mut FenOpts {
-        self.scid = scid;
-        self
-    }
+        let candidates = board.by_piece(color.rook()) & color.backrank();
 
-    fn castling_fen(&self, board: &Board, castling_rights: Bitboard) -> String {
-        let mut fen = String::with_capacity(4);
-
-        for color in Color::ALL {
-            let king = board.king_of(color);
-
-            let candidates = board.by_piece(color.rook()) & color.backrank();
-
-            for rook in (candidates & castling_rights).into_iter().rev() {
-                if !self.shredder
-                    && Some(rook) == candidates.first()
-                    && king.map_or(false, |k| rook < k)
-                {
-                    fen.push(color.fold_wb('Q', 'q'));
-                } else if !self.shredder
-                    && Some(rook) == candidates.last()
-                    && king.map_or(false, |k| k < rook)
-                {
-                    fen.push(color.fold_wb('K', 'k'));
+        for rook in (candidates & castling_rights).into_iter().rev() {
+            f.write_char(
+                if Some(rook) == candidates.first() && king.map_or(false, |k| rook < k) {
+                    color.fold_wb('Q', 'q')
+                } else if Some(rook) == candidates.last() && king.map_or(false, |k| k < rook) {
+                    color.fold_wb('K', 'k')
                 } else {
                     let file = rook.file();
-                    fen.push(color.fold_wb(file.char().to_ascii_uppercase(), file.char()));
-                }
-            }
-        }
-
-        if fen.is_empty() {
-            fen.push('-');
-        }
-
-        fen
-    }
-
-    /// Create an EPD such as
-    /// `rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -`.
-    pub fn epd(&self, setup: &dyn Setup) -> String {
-        let pockets = setup.pockets().map_or("".to_owned(), |p| {
-            if self.scid {
-                format!("/{}", p.fen())
-            } else {
-                format!("[{}]", p.fen())
-            }
-        });
-
-        let checks = setup.remaining_checks().map_or("".to_owned(), |r| {
-            if self.scid {
-                format!(
-                    " +{}+{}",
-                    3_u8.saturating_sub(u8::from(r.white)),
-                    3_u8.saturating_sub(u8::from(r.black))
-                )
-            } else {
-                format!(" {}", r)
-            }
-        });
-
-        let board = setup.board();
-
-        format!(
-            "{}{} {} {} {}{}",
-            board.board_fen(setup.promoted()),
-            pockets,
-            setup.turn().char(),
-            self.castling_fen(board, setup.castling_rights()),
-            setup
-                .ep_square()
-                .map_or("-".to_owned(), |sq| sq.to_string()),
-            checks
-        )
-    }
-
-    /// Create a FEN such as
-    /// `rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1`.
-    pub fn fen(&self, setup: &dyn Setup) -> String {
-        match setup.remaining_checks() {
-            Some(checks) if self.scid => {
-                let board = setup.board();
-
-                format!(
-                    "{}{} {} {} {} {} {} +{}+{}",
-                    board.board_fen(setup.promoted()),
-                    setup
-                        .pockets()
-                        .map_or("".to_owned(), |p| format!("/{}", p.fen())),
-                    setup.turn().char(),
-                    self.castling_fen(board, setup.castling_rights()),
-                    setup
-                        .ep_square()
-                        .map_or("-".to_owned(), |sq| sq.to_string()),
-                    setup.halfmoves(),
-                    setup.fullmoves(),
-                    3_u8.saturating_sub(u8::from(checks.white)),
-                    3_u8.saturating_sub(u8::from(checks.black))
-                )
-            }
-            _ => format!(
-                "{} {} {}",
-                self.epd(setup),
-                setup.halfmoves(),
-                setup.fullmoves()
-            ),
+                    color.fold_wb(file.char().to_ascii_uppercase(), file.char())
+                },
+            )?;
+            empty = false;
         }
     }
+
+    if empty {
+        f.write_char('-')?;
+    }
+
+    Ok(())
+}
+
+fn fmt_pockets(f: &mut fmt::Formatter<'_>, pockets: &ByColor<ByRole<u8>>) -> fmt::Result {
+    f.write_char('[')?;
+    for color in Color::ALL {
+        for role in Role::ALL {
+            let piece = Piece { color, role };
+            for _ in 0..*pockets.piece(piece) {
+                f.write_char(piece.char())?;
+            }
+        }
+    }
+    f.write_char(']')
+}
+
+fn fmt_epd(f: &mut fmt::Formatter<'_>, setup: &Setup) -> fmt::Result {
+    f.write_str(&setup.board.board_fen(setup.promoted))?;
+    if let Some(ref pockets) = setup.pockets {
+        fmt_pockets(f, pockets)?;
+    }
+    f.write_char(' ')?;
+    f.write_char(setup.turn.char())?;
+    f.write_char(' ')?;
+    fmt_castling(f, &setup.board, setup.castling_rights)?;
+    f.write_char(' ')?;
+    match setup.ep_square {
+        Some(ref ep_square) => Display::fmt(ep_square, f)?,
+        None => f.write_char('-')?,
+    }
+    if let Some(ref remaining_checks) = setup.remaining_checks {
+        f.write_char(' ')?;
+        Display::fmt(remaining_checks, f)?;
+    }
+    Ok(())
 }
 
 /// Errors that can occur when parsing a FEN.
@@ -227,7 +157,7 @@ pub enum ParseFenError {
     InvalidFullmoves,
 }
 
-impl fmt::Display for ParseFenError {
+impl Display for ParseFenError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str(match *self {
             ParseFenError::InvalidFen => "invalid fen",
@@ -258,6 +188,9 @@ fn parse_board_fen(board_fen: &[u8]) -> Result<(Board, Bitboard), ParseFenError>
         if ch == b'/' && file == 8 {
             file = 0;
             rank -= 1;
+            if rank < 0 {
+                return Err(ParseFenError::InvalidBoard);
+            }
         } else if (b'1'..=b'8').contains(&ch) {
             file += (ch - b'0') as i8;
             if file > 8 {
@@ -295,23 +228,34 @@ fn parse_remaining_checks(s: &[u8]) -> Option<ByColor<RemainingChecks>> {
         (Some(b""), Some(white_given), Some(black_given)) => {
             // format: +0+0
             ByColor {
-                white: RemainingChecks(3_u8.checked_sub(btoi::btou(white_given).ok()?)?),
-                black: RemainingChecks(3_u8.checked_sub(btoi::btoi(black_given).ok()?)?),
+                white: RemainingChecks::new(3_u32.checked_sub(btoi::btou(white_given).ok()?)?),
+                black: RemainingChecks::new(3_u32.checked_sub(btoi::btoi(black_given).ok()?)?),
             }
         }
         (Some(white), Some(black), None) => {
             // format: 3+3
             ByColor {
-                white: RemainingChecks(btoi::btou(white).ok()?),
-                black: RemainingChecks(btoi::btou(black).ok()?),
+                white: RemainingChecks::try_from(btoi::btou::<u32>(white).ok()?).ok()?,
+                black: RemainingChecks::try_from(btoi::btou::<u32>(black).ok()?).ok()?,
             }
         }
         _ => return None,
     })
 }
 
+fn parse_pockets(s: &[u8]) -> Option<ByColor<ByRole<u8>>> {
+    if s.len() > 64 {
+        return None;
+    }
+    let mut result = ByColor::<ByRole<u8>>::default();
+    for ch in s {
+        *result.piece_mut(Piece::from_char(char::from(*ch))?) += 1;
+    }
+    Some(result)
+}
+
 impl Board {
-    pub fn from_board_fen(board_fen: &[u8]) -> Result<Board, ParseFenError> {
+    pub fn from_ascii_board_fen(board_fen: &[u8]) -> Result<Board, ParseFenError> {
         Ok(parse_board_fen(board_fen)?.0)
     }
 
@@ -365,97 +309,28 @@ impl FromStr for Board {
     type Err = ParseFenError;
 
     fn from_str(board_fen: &str) -> Result<Board, ParseFenError> {
-        Board::from_board_fen(board_fen.as_bytes())
+        Board::from_ascii_board_fen(board_fen.as_bytes())
     }
 }
 
-impl fmt::Display for Board {
+impl Display for Board {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_str(&self.board_fen(Bitboard(0)))
     }
 }
 
-/// A parsed FEN.
-#[derive(Clone, Eq, PartialEq, Hash, Debug)]
-pub struct Fen {
-    pub board: Board,
-    pub promoted: Bitboard,
-    pub pockets: Option<Material>,
-    pub turn: Color,
-    pub castling_rights: Bitboard,
-    pub ep_square: Option<Square>,
-    pub remaining_checks: Option<ByColor<RemainingChecks>>,
-    pub halfmoves: u32,
-    pub fullmoves: NonZeroU32,
-}
-
-impl Setup for Fen {
-    fn board(&self) -> &Board {
-        &self.board
-    }
-    fn promoted(&self) -> Bitboard {
-        self.promoted
-    }
-    fn pockets(&self) -> Option<&Material> {
-        self.pockets.as_ref()
-    }
-    fn turn(&self) -> Color {
-        self.turn
-    }
-    fn castling_rights(&self) -> Bitboard {
-        self.castling_rights
-    }
-    fn ep_square(&self) -> Option<Square> {
-        self.ep_square
-    }
-    fn remaining_checks(&self) -> Option<&ByColor<RemainingChecks>> {
-        self.remaining_checks.as_ref()
-    }
-    fn halfmoves(&self) -> u32 {
-        self.halfmoves
-    }
-    fn fullmoves(&self) -> NonZeroU32 {
-        self.fullmoves
-    }
-}
-
-impl Default for Fen {
-    fn default() -> Fen {
-        Fen {
-            board: Board::default(),
-            promoted: Bitboard(0),
-            pockets: None,
-            turn: Color::White,
-            castling_rights: Bitboard::CORNERS,
-            ep_square: None,
-            remaining_checks: None,
-            halfmoves: 0,
-            fullmoves: NonZeroU32::new(1).unwrap(),
-        }
-    }
-}
+/// A FEN like `rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1`.
+#[derive(Clone, Eq, PartialEq, Hash, Debug, Default)]
+pub struct Fen(pub Setup);
 
 impl Fen {
     /// The FEN of the empty position `8/8/8/8/8/8/8/8 w - - 0 1`.
     pub fn empty() -> Fen {
-        Fen {
-            board: Board::empty(),
-            castling_rights: Bitboard(0),
-            ..Fen::default()
-        }
+        Fen(Setup::empty())
     }
 
-    /// Set up a [`Position`].
-    ///
-    /// # Errors
-    ///
-    /// Returns [`PositionError`] if the setup is not a legal position.
-    ///
-    /// [`FromSetup`]: super::FromSetup
-    /// [`Position`]: super::Position
-    /// [`PositionError`]: super::PositionError
-    pub fn position<P: FromSetup>(&self, mode: CastlingMode) -> Result<P, PositionError<P>> {
-        P::from_setup(self, mode)
+    pub fn from_position<P: Position>(pos: P, mode: EnPassantMode) -> Fen {
+        Fen(pos.into_setup(mode))
     }
 
     /// Parses a FEN or EPD.
@@ -477,7 +352,7 @@ impl Fen {
     /// # Ok::<_, Box<dyn std::error::Error>>(())
     /// ```
     pub fn from_ascii(fen: &[u8]) -> Result<Fen, ParseFenError> {
-        let mut result = Fen::empty();
+        let mut result = Setup::empty();
         let mut parts = fen
             .split(|ch| *ch == b' ' || *ch == b'_')
             .filter(|s| !s.is_empty());
@@ -512,9 +387,7 @@ impl Fen {
         result.promoted = promoted;
 
         if let Some(pocket_part) = pocket_part {
-            result.pockets = Some(
-                Material::from_ascii_fen(pocket_part).map_err(|_| ParseFenError::InvalidPocket)?,
-            );
+            result.pockets = Some(parse_pockets(pocket_part).ok_or(ParseFenError::InvalidPocket)?);
         }
 
         result.turn = match parts.next() {
@@ -605,22 +478,34 @@ impl Fen {
         if last_part.is_some() {
             Err(ParseFenError::InvalidFen)
         } else {
-            Ok(result)
+            Ok(Fen(result))
         }
     }
 
-    pub fn from_setup<S: Setup>(setup: &S) -> Fen {
-        Fen {
-            board: setup.board().clone(),
-            promoted: setup.promoted(),
-            pockets: setup.pockets().cloned(),
-            turn: setup.turn(),
-            castling_rights: setup.castling_rights(),
-            ep_square: setup.ep_square(),
-            remaining_checks: setup.remaining_checks().cloned(),
-            halfmoves: setup.halfmoves(),
-            fullmoves: setup.fullmoves(),
-        }
+    pub fn into_setup(self) -> Setup {
+        self.0
+    }
+
+    /// Set up a [`Position`]. See [`FromSetup`].
+    ///
+    /// # Errors
+    ///
+    /// Returns [`PositionError`] if the setup does not meet basic validity
+    /// requirements.
+    pub fn into_position<P: FromSetup>(self, mode: CastlingMode) -> Result<P, PositionError<P>> {
+        P::from_setup(self.0, mode)
+    }
+}
+
+impl From<Setup> for Fen {
+    fn from(setup: Setup) -> Fen {
+        Fen(setup)
+    }
+}
+
+impl From<Fen> for Setup {
+    fn from(fen: Fen) -> Setup {
+        fen.0
     }
 }
 
@@ -632,42 +517,93 @@ impl FromStr for Fen {
     }
 }
 
-impl fmt::Display for Fen {
+impl Display for Fen {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.write_str(&FenOpts::default().fen(self))
+        fmt_epd(f, &self.0)?;
+        write!(f, " {} {}", self.0.halfmoves, self.0.fullmoves)
     }
 }
 
-/// Create an EPD such as
-/// `rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -` with default
-/// [`FenOpts`].
-pub fn epd(setup: &dyn Setup) -> String {
-    FenOpts::default().epd(setup)
+/// An EPD like `rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq -`.
+#[derive(Clone, Eq, PartialEq, Hash, Debug, Default)]
+pub struct Epd(Setup);
+
+impl Epd {
+    pub fn empty() -> Epd {
+        Epd(Setup::empty())
+    }
+
+    pub fn from_ascii(epd: &[u8]) -> Result<Epd, ParseFenError> {
+        Ok(Epd::from(Fen::from_ascii(epd)?.into_setup()))
+    }
+
+    pub fn from_position<P: Position>(pos: P, mode: EnPassantMode) -> Epd {
+        Epd::from(pos.into_setup(mode))
+    }
+
+    pub fn into_setup(self) -> Setup {
+        self.0
+    }
+
+    pub fn into_position<P: FromSetup>(self, mode: CastlingMode) -> Result<P, PositionError<P>> {
+        P::from_setup(self.into_setup(), mode)
+    }
 }
 
-/// Create a FEN such as
-/// `rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1` with default
-/// [`FenOpts`].
-pub fn fen(setup: &dyn Setup) -> String {
-    FenOpts::default().fen(setup)
+impl From<Setup> for Epd {
+    fn from(mut setup: Setup) -> Epd {
+        setup.halfmoves = 0;
+        setup.fullmoves = NonZeroU32::new(1).unwrap();
+        Epd(setup)
+    }
+}
+
+impl From<Epd> for Setup {
+    fn from(Epd(setup): Epd) -> Setup {
+        setup
+    }
+}
+
+impl FromStr for Epd {
+    type Err = ParseFenError;
+
+    fn from_str(epd: &str) -> Result<Epd, ParseFenError> {
+        Epd::from_ascii(epd.as_bytes())
+    }
+}
+
+impl Display for Epd {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt_epd(f, &self.0)
+    }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::position::Chess;
+    use crate::{Chess, EnPassantMode, Position};
 
     #[test]
     fn test_legal_ep_square() {
         let original_epd = "4k3/8/8/8/3Pp3/8/8/3KR3 b - d3";
         let fen: Fen = original_epd.parse().expect("valid fen");
-        assert_eq!(epd(&fen), original_epd);
+        assert_eq!(
+            Epd::from(fen.clone().into_setup()).to_string(),
+            original_epd
+        );
 
         // The en passant square is not actually legal.
         let pos: Chess = fen
-            .position(CastlingMode::Standard)
+            .into_position(CastlingMode::Standard)
             .expect("legal position");
-        assert_eq!(epd(&pos), "4k3/8/8/8/3Pp3/8/8/3KR3 b - -");
+        assert_eq!(pos.maybe_ep_square(), Some(Square::D3));
+        assert_eq!(pos.pseudo_legal_ep_square(), Some(Square::D3));
+        assert_eq!(pos.legal_ep_square(), None);
+
+        assert_eq!(
+            Epd::from_position(pos, EnPassantMode::Legal).to_string(),
+            "4k3/8/8/8/3Pp3/8/8/3KR3 b - -"
+        );
     }
 
     #[test]
@@ -705,64 +641,65 @@ mod tests {
 
     #[test]
     fn test_pockets() {
-        let fen: Fen = "8/8/8/8/8/8/8/8[Q]".parse().expect("valid fen");
+        let setup = "8/8/8/8/8/8/8/8[Q]"
+            .parse::<Fen>()
+            .expect("valid fen")
+            .into_setup();
         assert_eq!(
-            fen.pockets()
-                .map_or(0, |p| p.by_piece(Color::White.queen())),
-            1
+            setup.pockets.map(|p| *p.piece(Color::White.queen())),
+            Some(1)
         );
     }
 
     #[test]
     fn test_lichess_promoted() {
-        let input = "rnbqk1nQ~/ppppp3/8/5p2/8/5N2/PPPPPPP1/RNBQKB1R/PPBR b KQq - 0 6";
-        let fen: Fen = input.parse().expect("valid fen");
-        assert_eq!(fen.promoted, Bitboard::from(Square::H8));
-        assert_eq!(FenOpts::default().scid(true).fen(&fen), input);
+        let setup = "rnbqk1nQ~/ppppp3/8/5p2/8/5N2/PPPPPPP1/RNBQKB1R/PPBR b KQq - 0 6"
+            .parse::<Fen>()
+            .expect("valid fen")
+            .into_setup();
+        assert_eq!(setup.promoted, Bitboard::from(Square::H8));
     }
 
     #[test]
     fn test_lichess_pockets() {
-        let input = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR/ w KQkq - 0 1";
-        let fen: Fen = input.parse().expect("valid fen");
-        assert_eq!(fen.pockets().map(|p| p.is_empty()), Some(true));
-        assert_eq!(FenOpts::default().scid(true).fen(&fen), input);
-    }
-
-    #[test]
-    fn test_shredder_fen() {
-        let pos = Chess::default();
-        assert_eq!(
-            FenOpts::default().shredder(true).fen(&pos),
-            "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w HAha - 0 1"
-        );
+        let setup = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR/ w KQkq - 0 1"
+            .parse::<Fen>()
+            .expect("valid fen")
+            .into_setup();
+        assert_eq!(setup.pockets, Some(Default::default()));
     }
 
     #[test]
     fn test_remaining_checks() {
-        let fen: Fen = "8/8/8/8/8/8/8/8 w - - 1+2 12 42"
-            .parse()
-            .expect("valid fen");
-        let expected = ByColor {
-            white: RemainingChecks(1),
-            black: RemainingChecks(2),
-        };
-        assert_eq!(fen.remaining_checks, Some(expected));
-        assert_eq!(fen.halfmoves, 12);
-        assert_eq!(fen.fullmoves.get(), 42);
+        let setup = "8/8/8/8/8/8/8/8 w - - 1+2 12 42"
+            .parse::<Fen>()
+            .expect("valid fen")
+            .into_setup();
+        assert_eq!(
+            setup.remaining_checks,
+            Some(ByColor {
+                white: RemainingChecks::new(1),
+                black: RemainingChecks::new(2),
+            })
+        );
+        assert_eq!(setup.halfmoves, 12);
+        assert_eq!(setup.fullmoves.get(), 42);
     }
 
     #[test]
     fn test_lichess_remaining_checks() {
-        let input = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 1 2 +0+0";
-        let fen: Fen = input.parse().expect("valid fen");
-        let expected = ByColor {
-            white: RemainingChecks(3),
-            black: RemainingChecks(3),
-        };
-        assert_eq!(fen.remaining_checks, Some(expected));
-        assert_eq!(fen.halfmoves, 1);
-        assert_eq!(fen.fullmoves.get(), 2);
-        assert_eq!(FenOpts::default().scid(true).fen(&fen), input);
+        let setup = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 1 2 +0+0"
+            .parse::<Fen>()
+            .expect("valid fen")
+            .into_setup();
+        assert_eq!(
+            setup.remaining_checks,
+            Some(ByColor {
+                white: RemainingChecks::new(3),
+                black: RemainingChecks::new(3),
+            })
+        );
+        assert_eq!(setup.halfmoves, 1);
+        assert_eq!(setup.fullmoves.get(), 2);
     }
 }
