@@ -68,19 +68,18 @@
 //! * `step`: Implements [`std::iter::Step`] for `Square`, `File`, and `Rank`.
 //!   Requires nightly Rust.
 
-#![doc(html_root_url = "https://docs.rs/shakmaty/0.20.7")]
+#![doc(html_root_url = "https://docs.rs/shakmaty/0.21.0")]
 #![forbid(unsafe_op_in_unsafe_fn)]
 #![warn(missing_debug_implementations)]
 #![cfg_attr(feature = "step", feature(step_trait))]
 #![cfg_attr(docs_rs, feature(doc_cfg))]
 
-mod board;
 mod color;
 mod magics;
-mod material;
 mod movelist;
 mod perft;
 mod position;
+mod role;
 mod setup;
 mod square;
 mod types;
@@ -88,6 +87,7 @@ mod util;
 
 pub mod attacks;
 pub mod bitboard;
+pub mod board;
 pub mod fen;
 pub mod san;
 pub mod uci;
@@ -99,16 +99,16 @@ pub mod variant;
 
 pub use crate::{
     bitboard::Bitboard,
-    board::{Board, Pieces},
-    color::{ByColor, ByColorIter, Color, ParseColorError},
-    material::{Material, MaterialSide, ParseMaterialError},
+    board::Board,
+    color::{ByColor, Color, ParseColorError},
     movelist::MoveList,
     perft::perft,
     position::{
         Chess, FromSetup, Outcome, ParseOutcomeError, PlayError, Position, PositionError,
         PositionErrorKinds,
     },
+    role::{ByRole, Role},
     setup::{Castles, Setup},
     square::{File, ParseSquareError, Rank, Square},
-    types::{CastlingMode, CastlingSide, Move, Piece, RemainingChecks, Role},
+    types::{CastlingMode, CastlingSide, EnPassantMode, Move, Piece, RemainingChecks},
 };
