@@ -15,178 +15,17 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 use std::{
-    char,
     fmt::{self, Write as _},
     num,
 };
 
-pub use self::Role::{Bishop, King, Knight, Pawn, Queen, Rook};
 use crate::{
     color::{ByColor, Color},
+    role::Role,
     square::{File, Square},
     util::overflow_error,
 };
 
-/// Piece types: `Pawn`, `Knight`, `Bishop`, `Rook`, `Queen`, `King`.
-///
-/// # Examples
-///
-/// ```
-/// use shakmaty::Role;
-///
-/// // Piece types are indexed from 1 to 6.
-/// assert_eq!(u32::from(Role::Pawn), 1);
-/// assert_eq!(u32::from(Role::King), 6);
-/// ```
-#[allow(missing_docs)]
-#[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Debug, Hash)]
-pub enum Role {
-    Pawn = 1,
-    Knight = 2,
-    Bishop = 3,
-    Rook = 4,
-    Queen = 5,
-    King = 6,
-}
-
-impl Role {
-    /// Gets the piece type from its English letter.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use shakmaty::Role;
-    ///
-    /// assert_eq!(Role::from_char('K'), Some(Role::King));
-    /// assert_eq!(Role::from_char('n'), Some(Role::Knight));
-    ///
-    /// assert_eq!(Role::from_char('X'), None);
-    /// ```
-    pub fn from_char(ch: char) -> Option<Role> {
-        match ch {
-            'P' | 'p' => Some(Role::Pawn),
-            'N' | 'n' => Some(Role::Knight),
-            'B' | 'b' => Some(Role::Bishop),
-            'R' | 'r' => Some(Role::Rook),
-            'Q' | 'q' => Some(Role::Queen),
-            'K' | 'k' => Some(Role::King),
-            _ => None,
-        }
-    }
-
-    /// Gets a [`Piece`] of the given color.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use shakmaty::{Color, Role};
-    ///
-    /// assert_eq!(Role::King.of(Color::Black), Color::Black.king());
-    /// ```
-    #[inline]
-    pub fn of(self, color: Color) -> Piece {
-        Piece { color, role: self }
-    }
-
-    /// Gets the English letter for the piece type.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use shakmaty::Role;
-    ///
-    /// assert_eq!(Role::Rook.char(), 'r');
-    /// ```
-    pub fn char(self) -> char {
-        match self {
-            Role::Pawn => 'p',
-            Role::Knight => 'n',
-            Role::Bishop => 'b',
-            Role::Rook => 'r',
-            Role::Queen => 'q',
-            Role::King => 'k',
-        }
-    }
-
-    /// Gets the uppercase English letter for the piece type.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use shakmaty::Role;
-    ///
-    /// assert_eq!(Role::Rook.upper_char(), 'R');
-    /// ```
-    pub fn upper_char(self) -> char {
-        match self {
-            Role::Pawn => 'P',
-            Role::Knight => 'N',
-            Role::Bishop => 'B',
-            Role::Rook => 'R',
-            Role::Queen => 'Q',
-            Role::King => 'K',
-        }
-    }
-
-    /// `Pawn`, `Knight`, `Bishop`, `Rook`, `Queen`, and `King`, in this order.
-    pub const ALL: [Role; 6] = [Pawn, Knight, Bishop, Rook, Queen, King];
-}
-
-macro_rules! int_from_role_impl {
-    ($($t:ty)+) => {
-        $(impl From<Role> for $t {
-            #[inline]
-            fn from(role: Role) -> $t {
-                role as $t
-            }
-        })+
-    }
-}
-
-int_from_role_impl! { u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
-
-macro_rules! nonzero_int_from_role_impl {
-    ($($t:ty)+) => {
-        $(impl From<Role> for $t {
-            #[inline]
-            fn from(role: Role) -> $t {
-                <$t>::new(role.into()).expect("nonzero role discriminant")
-            }
-        })+
-    }
-}
-
-nonzero_int_from_role_impl! {
-    num::NonZeroU8 num::NonZeroI8
-    num::NonZeroU16 num::NonZeroI16
-    num::NonZeroU32 num::NonZeroI32
-    num::NonZeroU64 num::NonZeroI64
-    num::NonZeroUsize num::NonZeroIsize
-}
-
-macro_rules! try_role_from_int_impl {
-    ($($t:ty)+) => {
-        $(impl std::convert::TryFrom<$t> for Role {
-            type Error = num::TryFromIntError;
-
-            #[inline]
-            fn try_from(value: $t) -> Result<Role, Self::Error> {
-                Ok(match value {
-                    1 => Role::Pawn,
-                    2 => Role::Knight,
-                    3 => Role::Bishop,
-                    4 => Role::Rook,
-                    5 => Role::Queen,
-                    6 => Role::King,
-                    _ => return Err(overflow_error()),
-                })
-            }
-        })+
-    }
-}
-
-try_role_from_int_impl! { u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
-
 /// A piece with [`Color`] and [`Role`].
 #[allow(missing_docs)]
 #[derive(Copy, Clone, Eq, PartialEq, Debug, Hash)]
@@ -262,7 +101,7 @@ impl Move {
     pub fn capture(&self) -> Option<Role> {
         match *self {
             Move::Normal { capture, .. } => capture,
-            Move::EnPassant { .. } => Some(Pawn),
+            Move::EnPassant { .. } => Some(Role::Pawn),
             _ => None,
         }
     }
@@ -471,6 +310,22 @@ impl CastlingMode {
     }
 }
 
+/// When to include the en passant square.
+#[derive(Copy, Clone, Eq, PartialEq, Debug, Hash)]
+pub enum EnPassantMode {
+    /// Only if there is a fully legal en passant move.
+    Legal,
+    /// Only when a pawn has been advanced by two steps and there is an
+    /// enemy pawn next to it.
+    ///
+    /// Follows the X-FEN specification.
+    PseudoLegal,
+    /// Whenever a pawn has been advanced by two steps.
+    ///
+    /// Follows the FEN specification.
+    Always,
+}
+
 #[cfg(test)]
 mod tests {
     use std::mem;
@@ -494,8 +349,24 @@ mod tests {
 
 /// The number of checks the respective side needs to give in order to win
 /// (in a game of Three-Check).
+///
+/// # Examples
+///
+/// ```
+/// use shakmaty::{ByColor, RemainingChecks};
+///
+/// let mut remaining_checks = ByColor::<RemainingChecks>::default();
+/// assert_eq!(remaining_checks.white, RemainingChecks::new(3));
+/// assert_eq!(remaining_checks.black, RemainingChecks::new(3));
+///
+/// for _ in 0..5 {
+///     remaining_checks.white = remaining_checks.white.saturating_sub(1);
+/// }
+///
+/// assert!(remaining_checks.white.is_zero());
+/// ```
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Debug, Hash)]
-pub struct RemainingChecks(pub u8);
+pub struct RemainingChecks(u32);
 
 impl Default for RemainingChecks {
     fn default() -> RemainingChecks {
@@ -504,13 +375,23 @@ impl Default for RemainingChecks {
 }
 
 impl RemainingChecks {
+    /// Constructs a new [`RemainingChecks`] value.
+    ///
+    /// # Panics
+    ///
+    /// Panics if `n > 3`.
+    pub fn new(n: u32) -> RemainingChecks {
+        assert!(n <= 3);
+        RemainingChecks(n)
+    }
+
     pub fn is_zero(self) -> bool {
         self.0 == 0
     }
 
     #[must_use]
-    pub fn minus_one(self) -> RemainingChecks {
-        RemainingChecks(self.0.saturating_sub(1))
+    pub fn saturating_sub(self, n: u32) -> RemainingChecks {
+        RemainingChecks(self.0.saturating_sub(n))
     }
 }
 
@@ -527,6 +408,26 @@ macro_rules! int_from_remaining_checks_impl {
 
 int_from_remaining_checks_impl! { u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
 
+macro_rules! try_remaining_checks_from_int_impl {
+    ($($t:ty)+) => {
+        $(impl std::convert::TryFrom<$t> for RemainingChecks {
+            type Error = num::TryFromIntError;
+
+            #[inline]
+            fn try_from(value: $t) -> Result<RemainingChecks, Self::Error> {
+                let n = u32::try_from(value)?;
+                if n <= 3 {
+                    Ok(RemainingChecks::new(n))
+                } else {
+                    Err(overflow_error())
+                }
+            }
+        })+
+    }
+}
+
+try_remaining_checks_from_int_impl! { u8 i8 u16 i16 u32 i32 u64 i64 usize isize }
+
 impl fmt::Display for ByColor<RemainingChecks> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{}+{}", self.white.0, self.black.0)
