@@ -34,9 +34,9 @@
 use std::{cell::Cell, num::NonZeroU32, ops::BitXorAssign};
 
 use crate::{
-    Bitboard, Board, ByColor, Castles, CastlingMode, CastlingSide, Chess, Color, File, FromSetup,
-    Material, Move, MoveList, Outcome, Piece, Position, PositionError, RemainingChecks, Role,
-    Setup, Square,
+    color::ByColor, Bitboard, Board, ByRole, Castles, CastlingMode, CastlingSide, Chess, Color,
+    EnPassantMode, File, FromSetup, Move, MoveList, Outcome, Piece, Position, PositionError,
+    RemainingChecks, Role, Setup, Square,
 };
 
 /// Integer type that can be returned as a Zobrist hash.
@@ -75,8 +75,8 @@ macro_rules! zobrist_value_impl {
                 EN_PASSANT_FILE_MASKS[usize::from(file)] as $t
             }
 
-            fn zobrist_for_remaining_checks(color: Color, RemainingChecks(remaining): RemainingChecks) -> $t {
-                if remaining < 3 {
+            fn zobrist_for_remaining_checks(color: Color, remaining: RemainingChecks) -> $t {
+                if remaining < RemainingChecks::default() {
                     REMAINING_CHECKS_MASKS[usize::from(remaining) + color.fold_wb(0, 3)] as $t
                 } else {
                     <$t>::default()
@@ -258,7 +258,7 @@ impl<P: Default, V: ZobristValue> Default for Zobrist<P, V> {
 }
 
 impl<P: FromSetup + Position, V: ZobristValue> FromSetup for Zobrist<P, V> {
-    fn from_setup(setup: &dyn Setup, mode: CastlingMode) -> Result<Self, PositionError<Self>> {
+    fn from_setup(setup: Setup, mode: CastlingMode) -> Result<Self, PositionError<Self>> {
         match P::from_setup(setup, mode) {
             Ok(pos) => Ok(Zobrist::new(pos)),
             Err(err) => Err(PositionError {
@@ -269,24 +269,24 @@ impl<P: FromSetup + Position, V: ZobristValue> FromSetup for Zobrist<P, V> {
     }
 }
 
-impl<P: Setup, V: ZobristValue> Setup for Zobrist<P, V> {
+impl<P: Position + ZobristHash, V: ZobristValue> Position for Zobrist<P, V> {
     fn board(&self) -> &Board {
         self.pos.board()
     }
     fn promoted(&self) -> Bitboard {
         self.pos.promoted()
     }
-    fn pockets(&self) -> Option<&Material> {
+    fn pockets(&self) -> Option<&ByColor<ByRole<u8>>> {
         self.pos.pockets()
     }
     fn turn(&self) -> Color {
         self.pos.turn()
     }
-    fn castling_rights(&self) -> Bitboard {
-        self.pos.castling_rights()
+    fn castles(&self) -> &Castles {
+        self.pos.castles()
     }
-    fn ep_square(&self) -> Option<Square> {
-        self.pos.ep_square()
+    fn maybe_ep_square(&self) -> Option<Square> {
+        self.pos.maybe_ep_square()
     }
     fn remaining_checks(&self) -> Option<&ByColor<RemainingChecks>> {
         self.pos.remaining_checks()
@@ -297,9 +297,9 @@ impl<P: Setup, V: ZobristValue> Setup for Zobrist<P, V> {
     fn fullmoves(&self) -> NonZeroU32 {
         self.pos.fullmoves()
     }
-}
-
-impl<P: Position + ZobristHash, V: ZobristValue> Position for Zobrist<P, V> {
+    fn into_setup(self, mode: EnPassantMode) -> Setup {
+        self.pos.into_setup(mode)
+    }
     fn legal_moves(&self) -> MoveList {
         self.pos.legal_moves()
     }
@@ -324,9 +324,6 @@ impl<P: Position + ZobristHash, V: ZobristValue> Position for Zobrist<P, V> {
     fn king_attackers(&self, square: Square, attacker: Color, occupied: Bitboard) -> Bitboard {
         self.pos.king_attackers(square, attacker, occupied)
     }
-    fn castles(&self) -> &Castles {
-        self.pos.castles()
-    }
     fn is_variant_end(&self) -> bool {
         self.pos.is_variant_end()
     }
@@ -354,7 +351,7 @@ impl<P: Position + ZobristHash, V: ZobristValue> Position for Zobrist<P, V> {
 
 fn hash_board<V: ZobristValue>(board: &Board) -> V {
     let mut zobrist = V::default();
-    for (sq, piece) in board.pieces() {
+    for (sq, piece) in board.clone() {
         zobrist ^= V::zobrist_for_piece(sq, piece);
     }
     zobrist
@@ -370,7 +367,7 @@ fn hash_position<P: Position, V: ZobristValue>(pos: &P) -> V {
     if let Some(pockets) = pos.pockets() {
         for (color, pocket) in pockets.as_ref().zip_color() {
             for role in Role::ALL {
-                zobrist ^= V::zobrist_for_pocket(color, role, pocket.by_role(role));
+                zobrist ^= V::zobrist_for_pocket(color, role, *pocket.get(role));
             }
         }
     }
@@ -388,7 +385,7 @@ fn hash_position<P: Position, V: ZobristValue>(pos: &P) -> V {
         }
     }
 
-    if let Some(sq) = pos.ep_square() {
+    if let Some(sq) = pos.legal_ep_square() {
         zobrist ^= V::zobrist_for_en_passant_file(sq.file());
     }
 
@@ -451,7 +448,7 @@ mod tests {
             let pos: Chess = fen
                 .parse::<Fen>()
                 .expect("valid fen")
-                .position(CastlingMode::Standard)
+                .into_position(CastlingMode::Standard)
                 .expect("legal position");
 
             assert_eq!(pos.zobrist_hash::<u64>(), expected, "{}", fen);
